<?php

namespace App\Classes;

use App\Classes\BaseHandler;

class LatestListenHandler extends BaseHandler
{
  protected int $cacheTTL = 60;

  public function __construct()
  {
    parent::__construct();
    $this->initializeCache();
  }

  public function handleRequest(): void
  {
    $data = $this->getLatestListen();

    if (!$data) {
      $this->sendResponse(["message" => "No recent tracks found"], 404);

      return;
    }

    $this->sendResponse($data);
  }

  public function getLatestListen(): ?array
  {
    try {
      $cachedData = $this->cache ? $this->cache->get("latest_listen") : null;

      if ($cachedData) return json_decode($cachedData, true);

      $data = $this->makeRequest("GET", "optimized_latest_listen?select=*");

      if (!is_array($data) || empty($data[0])) return null;

      $latestListen = $this->formatLatestListen($data[0]);

      if ($this->cache) $this->cache->set("latest_listen", json_encode($latestListen), $this->cacheTTL);

      return $latestListen;
    } catch (\Exception $e) {
      error_log("LatestListenHandler::getLatestListen error: " . $e->getMessage());
      return null;
    }
  }

  private function formatLatestListen(array $latestListen): array
  {
    $emoji = $latestListen["artist_emoji"] ?? ($latestListen["genre_emoji"] ?? "🎧");
    $trackName = htmlspecialchars($latestListen["track_name"] ?? "Unknown Track", ENT_QUOTES, "UTF-8");
    $artistName = htmlspecialchars($latestListen["artist_name"] ?? "Unknown Artist", ENT_QUOTES, "UTF-8");
    $url = htmlspecialchars($latestListen["url"] ?? "/", ENT_QUOTES, "UTF-8");

    return [
      "content" => sprintf(
        '%s %s by <a href="%s">%s</a>',
        $emoji,
        $trackName,
        $url,
        $artistName
      ),
    ];
  }
}
