import * as cheerio from 'cheerio';

export default {
  convertRelativeLinks: (htmlContent, domain) => {
    if (!htmlContent || !domain) return htmlContent;

    const $ = cheerio.load(htmlContent);

    $("a[href]").each((_, link) => {
      const $link = $(link);
      let href = $link.attr("href");

      if (href.startsWith("#")) {
        $link.replaceWith($("<span>").text($link.text()));
      } else if (!href.startsWith("http://") && !href.startsWith("https://")) {
        const normalizedDomain = domain.replace(/\/$/, '');
        const normalizedHref = href.replace(/^\/+/, '');
        $link.attr("href", `${normalizedDomain}/${normalizedHref}`);
      }
    });

    return $.html();
  },
  generatePermalink: (url, baseUrl) => {
    if (url?.includes("http") || !baseUrl) return url
    return `${baseUrl}${url}`
  },
  getRemoteFileSize: async (url) => {
    try {
      const response = await fetch(url, { method: "HEAD" });

      if (!response.ok) return 0;

      const contentLength = response.headers.get("content-length");

      if (!contentLength) return 0;

      return parseInt(contentLength, 10);
    } catch (error) {
      return 0;
    }
  }
}
